#include <avr/io.h>

//Hier kann man die Funktion zum Kompilieren setzen.
#define digitfunc 1


/********************************************************************************************/
/********************************************************************************************/

//Bitstore, store bit to T-Flag
#define bst(byte,i) asm volatile("bst %0, %1\n\t" : "+r"(byte) : "M"(i))

/********************************************************************************************/
//Bitload, load bit from T-Flag
#define bld(byte,i) asm volatile("bld %0, %1\n\t" : "+r"(byte) : "M"(i))

/********************************************************************************************/
//Bitcopy, source will be changed!
#define bcpf(source, dest, mask)						\
	asm volatile(	"andi %[src], %[msk]		\n\t"	\
					"andi %[dst], ~%[msk]		\n\t"	\
					"or   %[dst], %[src]		\n\t"	\
					: [dst] "+d"(dest) : [src] "r"(source), [msk] "M"(mask))

/********************************************************************************************/
/********************************************************************************************/


uint8_t lcd[8];

#if digitfunc == 1
void SetDigit(uint8_t digit, uint8_t value)
{
	uint8_t d1, d2, d3, d4;
	uint8_t* lcd_addr = lcd;
	asm volatile(	"lsr %[digit]				\n\t" 
					"adc %A[addr], __zero_reg__	\n\t" //Wenn bit 0 von digit 1 war, addr um eins erhhen
					"adc %B[addr], __zero_reg__	\n\t"
					"ldd %[d1],%a[addr]+0		\n\t"
					"ldd %[d2],%a[addr]+2		\n\t"
					"ldd %[d3],%a[addr]+4		\n\t"
					"ldd %[d4],%a[addr]+6		\n\t"
					: [d1] "=r"(d1), [d2] "=r"(d2), [d3] "=r"(d3), [d4] "=r"(d4),
					  [addr] "+b"(lcd_addr), [digit] "+r"(digit));

	switch(digit)
	{
		case 0:
			bst(value, 0);
			bld(d3, 6);
			break;
		case 1:
			bst(value, 2);
			bld(d2, 7);
			bst(value, 3);
			bld(d3, 0);
			bst(value, 4);
			bld(d3, 1);
			bst(value, 5);
			bld(d1, 7);
			bst(value, 6);
			bld(d1, 6);
			bcpf(value, d2, 0x03);
			break;
		case 2:
			bst(value, 0);
			bld(d1, 3);
			bst(value, 1);
			bld(d1, 4);
			bst(value, 2);
			bld(d4, 7);
			bst(value, 3);
			bld(d2, 4);
			bst(value, 4);
			bld(d2, 5);
			bst(value, 5);
			bld(d1, 2);
			bst(value, 6);
			bld(d2, 3);
			break;
		case 3:
			bst(value, 0);
			bld(d3, 4);
			bst(value, 5);
			bld(d3, 5);
			bst(value, 6);
			bld(d1, 5);
			value <<= 1;
			bcpf(value, d4, 60);
			break;
			
	}

	asm volatile(	"std %a[addr]+0, %[d1]	\n\t"
					"std %a[addr]+2, %[d2]	\n\t"
					"std %a[addr]+4, %[d3]	\n\t"
					"std %a[addr]+6, %[d4]	\n\t"
					: : [d1] "r"(d1), [d2] "r"(d2), [d3] "r"(d3), [d4] "r"(d4),
					  [addr] "b"(lcd_addr) : "memory");

}
#elif digitfunc == 2
void SetDigit(uint8_t digit, uint8_t value)
{
	switch(digit >> 1)
	{
		case 0:
			bst(value, 0);
			bld(lcd[4+(digit&1)], 6);
			break;
		case 1:
			bst(value, 2);
			bld(lcd[2+(digit&1)], 7);
			bst(value, 3);
			bld(lcd[4+(digit&1)], 0);
			bst(value, 4);
			bld(lcd[4+(digit&1)], 1);
			bst(value, 5);
			bld(lcd[digit&1], 7);
			bst(value, 6);
			bld(lcd[digit&1], 6);
			bcpf(value, lcd[2+(digit&1)], 0x03);
			break;
		case 2:
			bst(value, 0);
			bld(lcd[digit&1], 3);
			bst(value, 1);
			bld(lcd[digit&1], 4);
			bst(value, 2);
			bld(lcd[6+(digit&1)], 7);
			bst(value, 3);
			bld(lcd[2+(digit&1)], 4);
			bst(value, 4);
			bld(lcd[2+(digit&1)], 5);
			bst(value, 5);
			bld(lcd[digit&1], 2);
			bst(value, 6);
			bld(lcd[2+(digit&1)], 3);
			break;
		case 3:
			bst(value, 0);
			bld(lcd[4+(digit&1)], 4);
			bst(value, 5);
			bld(lcd[4+(digit&1)], 5);
			bst(value, 6);
			bld(lcd[digit&1], 5);
			value <<= 1;
			bcpf(value, lcd[6+(digit&1)], 60);
			break;
			
	}
}
#else
void SetDigit(uint8_t digit, uint8_t value)
{
	uint8_t d1, d2, d3, d4;
	uint8_t* lcd_addr = lcd;

	if(digit & 0x01)
		lcd_addr++;
	digit >>=1;	
	d1 = lcd_addr[0];
	d2 = lcd_addr[2];
	d3 = lcd_addr[4];
	d4 = lcd_addr[6];

	switch(digit)
	{
		case 0:
			bst(value, 0);
			bld(d3, 6);
			break;
		case 1:
			bst(value, 2);
			bld(d2, 7);
			bst(value, 3);
			bld(d3, 0);
			bst(value, 4);
			bld(d3, 1);
			bst(value, 5);
			bld(d1, 7);
			bst(value, 6);
			bld(d1, 6);
			bcpf(value, d2, 0x03);
			break;
		case 2:
			bst(value, 0);
			bld(d1, 3);
			bst(value, 1);
			bld(d1, 4);
			bst(value, 2);
			bld(d4, 7);
			bst(value, 3);
			bld(d2, 4);
			bst(value, 4);
			bld(d2, 5);
			bst(value, 5);
			bld(d1, 2);
			bst(value, 6);
			bld(d2, 3);
			break;
		case 3:
			bst(value, 0);
			bld(d3, 4);
			bst(value, 5);
			bld(d3, 5);
			bst(value, 6);
			bld(d1, 5);
			value <<= 1;
			bcpf(value, d4, 60);
			break;
			
	}

	lcd_addr[0] = d1;
	lcd_addr[2] = d2;
	lcd_addr[4] = d3;
	lcd_addr[6] = d4;
}
#endif

int main(void)
{
	SetDigit(3, 0xFF);
	return 0;
}

